/*
 *  threedimensions.cpp
 *  GL
 *
 *  Created by NYU User on 11/24/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */
#include <iostream>
#include <cassert>	//for assert

#include <GLTools.h>			//OpenGL Toolkit
#include <GLShaderManager.h>	//Shader Manager Class
#include <GLFrustum.h>
#include <GLMatrixStack.h>
#include <GLFrame.h>

GLShaderManager shaderManager;
GLTriangleBatch torus;
GLFrustum frustum;
GLMatrixStack projection;
GLMatrixStack modelView;
GLFrame camera;

void SetupRC()
{
	shaderManager.InitializeStockShaders();
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);	//blue background
	gltMakeTorus(torus, 1.0f, 1.0f / 3.0f, 40, 40);	//in XY plane
	
	//Don't display the interior of the torus.
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
}

void reshape(GLsizei width, GLsizei height)
{
	std::cerr << "reshape(" << width << ", " << height << ")\n";
	//Fill the entire iPhone screen.
	glViewport(0, 0, width, height);
	
	//Make closer objects look bigger, farther smaller.
	
	frustum.SetPerspective(
		35.0f,		//vertical field of view in degrees
		static_cast<GLfloat>(width) / height,
		1.0f,		//nearest visible distance
		10.0f		//farthest
	);
	
	projection.LoadMatrix(frustum.GetProjectionMatrix());
}

void showLocation()
{
	M3DVector3f location;
	camera.GetOrigin(location);
	
	std::cout << "camera location == ("
		<< location[0] << ", "
		<< location[1] << ", "
		<< location[2] << ")\n";
}

void up()
{
	camera.MoveUp(-.1f);
	showLocation();
}

void down()
{
	camera.MoveUp(.1f);
	showLocation();
}

void left()
{
	camera.MoveRight(-.1f);
	showLocation();
}

void right()
{
	camera.MoveRight(.1f);
	showLocation();
}

void forward()
{
	camera.MoveForward(.1f);
	showLocation();
}

void backward()
{
	camera.MoveForward(-.1f);
	showLocation();
}
 
void panUp()    {camera.RotateLocalX(m3dDegToRad( 1.0f));}
void panDown()  {camera.RotateLocalX(m3dDegToRad(-1.0f));}

void panLeft()  {camera.RotateLocalY(m3dDegToRad(-1.0f));}
void panRight() {camera.RotateLocalY(m3dDegToRad( 1.0f));}

void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	modelView.PushMatrix();

	M3DMatrix44f m;
	camera.GetCameraMatrix(m);
	modelView.LoadMatrix(m);

	//A positive value of z would put the torus *behind* us
	//and we wouldn't see it.
	modelView.Translate(0.0f, 0.0f, -7.0f);
	
	static const M3DVector4f lightPosition = {0.0f, 10.0f, 5.0f, 1.0f};
	static const GLfloat red[] = {1.0f, 0.0f, 0.0f, 1.0f};
	
	shaderManager.UseStockShader(
		GLT_SHADER_POINT_LIGHT_DIFF,
		modelView.GetMatrix(),
		projection.GetMatrix(),
		lightPosition,
		red
	);
	
	torus.Draw();
	modelView.PopMatrix();
}
